@@ -574,7 +574,7 @@ class Connection:
     def get_dataset_indexes_url(self, dataset_id):
         return self.get_dataset_url(dataset_id) + "/indexes"
 
-    def restart_node(self, force=False, **kwargs):
+    def restart_node(self, **kwargs):
         """
             restarts the node via the admin api, it returns a dict {"message": "OK"} if successful
         """
@@ -583,8 +583,21 @@ class Connection:
 
         postdata = {"operation": "restart"}
 
-        if force is True:
-            postdata["force"] = True
+        _allowable_response_status_codes = [200]
+        restart_response = self.do_post_request(url, allowable_response_status_codes=_allowable_response_status_codes,
+                                                data=postdata, **kwargs)
+
+        validate_response_is_ok(restart_response, allowable_response_status_codes=_allowable_response_status_codes)
+        return utils.parse_json_response(restart_response)
+
+    def reset_node(self, **kwargs):
+        """
+            resets the node via the admin api, it returns a dict {"message": "OK"} if successful
+        """
+
+        url = self.admin_url
+
+        postdata = {"operation": "reset"}
 
         _allowable_response_status_codes = [200]
         restart_response = self.do_post_request(url, allowable_response_status_codes=_allowable_response_status_codes,
