@@ -1,6 +1,7 @@
 # Copyright (C) Bouvet ASA - All Rights Reserved.
 # Unauthorized copying of this file, via any medium is strictly prohibited.
 import copy
+import json
 import logging
 
 import io
@@ -9,7 +10,7 @@ import textwrap
 import time
 
 from . import entity_json
-from .exceptions import ConfigUploadFailed
+from .exceptions import ConfigUploadFailed, ExceptionWithResponse
 from .entitybase import EntityBase
 from .pump import Pump
 
@@ -103,10 +104,12 @@ class Pipe(EntityBase):
     def runtime(self):
         return copy.deepcopy(self._raw_jsondata["runtime"])
 
-    def get_entities(self, since=None, limit=None, **kwargs):
-        """This returns a generator for iterating over the entities of the source of this pipe.
+    def get_entities(self, since=None, limit=None, stage=None, **kwargs):
+        """This returns a generator for iterating over the entities of the specified stage
+        of this pipe.
         :param since:
         :param limit: The maximum number of entities to return. A value of None means return all the entities.
+        :param stage:
         """
         url = self._connection.get_pipe_entities_url(self.id)
 
@@ -115,6 +118,8 @@ class Pipe(EntityBase):
             params["since"] = since
         if limit is not None:
             params["limit"] = limit
+        if stage is not None:
+            params["stage"] = stage
 
         # The entities are streamed from the server, so we have to enable streaming here.
         response = self._connection.do_get_request(url, allowable_response_status_codes=[200],
@@ -196,7 +201,17 @@ class Pipe(EntityBase):
     def wait_for_pipe_to_be_deployed(self, timeout=300, **kwargs):
         starttime = time.monotonic()
         while True:
-            self.update_raw_jsondata_from_url(self._connection.get_pipe_url(self.id), **kwargs)
+            try:
+                self.update_raw_jsondata_from_url(self._connection.get_pipe_url(self.id), **kwargs)
+            except ExceptionWithResponse as e:
+                elapsetime = time.monotonic() - starttime
+                if elapsetime > timeout:
+                    raise AssertionError(
+                        f"Timed out while waiting for the config of the pipe '{self.id}' to be deployed. "
+                        f"exception: {e}")
+                time.sleep(1)
+                continue
+
             if self.runtime["state"] != "Deploying":
                 return
             elapsetime = time.monotonic() - starttime
@@ -205,3 +220,11 @@ class Pipe(EntityBase):
                     f"Timed out while waiting for the config of the pipe '{self.id}' to be deployed. "
                     f"self._raw_jsondata: {textwrap.indent(pprint.pformat(self._raw_jsondata), prefix='    ')}")
             time.sleep(1)
+
+    def preview(self, pipe_config, source_entity):
+        response = self._connection.do_post_request(self._connection.get_pipe_url(self.id) + "/preview",
+                                                    allowable_response_status_codes=[200],
+                                                    data={
+                                                        "pipe-config": json.dumps(pipe_config),
+                                                        "source": json.dumps(source_entity)})
+        return response.json()
