@@ -90,7 +90,7 @@ class Pump(EntityBase):
                                   self.id,
                                   elapsedtime,
                                   textwrap.indent(pprint.pformat(self.runtime["progress"]), prefix="    "),
-                                  self.runtime["last-seen"],
+                                  self.runtime.get("last-seen"),
                               ))
                     else:
                         print("Waiting for the pipe '%s' to finish running (elapsed time: %.2fs). "
@@ -98,7 +98,7 @@ class Pump(EntityBase):
                                   self.id,
                                   elapsedtime,
                                   self.runtime["progress"]["processed"],
-                                  self.runtime["last-seen"],
+                                  self.runtime.get("last-seen"),
                               ))
                     time_of_last_log_output = current_time
 
@@ -296,6 +296,35 @@ class Pump(EntityBase):
         """
         self.run_operation("stop")
 
+    def start_rescan(self, allow_already_running_pump=True):
+        """
+        Starts a rescan-run.
+        :param allow_already_running_pump: If this  flag is set to False, this method will fail with a
+                                           "PumpIsAlreadyRunning"-exception if if the pump was already
+                                           running.
+        """
+        response = self.run_operation("start-rescan", allowable_response_status_codes=frozenset([200, 409]))
+
+        if response.status_code == 409:
+            self.update_raw_jsondata(response.content.decode("utf-8"))
+            if allow_already_running_pump:
+                return False
+            else:
+                raise PumpIsAlreadyRunning(self)
+
+        validate_response_is_ok(response, 200)
+        self.update_raw_jsondata(response.content.decode("utf-8"))
+        return True
+
+    def reset_rescan(self):
+        """
+        Resets the rescan-run state.
+        """
+        response = self.run_operation("reset-rescan", allowable_response_status_codes=frozenset([200]))
+        validate_response_is_ok(response, 200)
+        self.update_raw_jsondata(response.content.decode("utf-8"))
+        return True
+
     @property
     def supported_operations(self):
         return self._raw_jsondata["runtime"]["supported-operations"]
